using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using SlimDX;

namespace MMDFileParser.XParser {
    public class XParser
    {
        private static XParser t = new XParser();

        public static XObj getXObject(StringData d)
        {
            XObj root = _getXObject(d);
            root.child = root.child.Where(x => x != null).ToList();

            return root;
        }

        private static XObj _getXObject(StringData d)
        {
            List<XObj> child = new List<XObj>();

            // get Object from d
            XObj parent = InvokeMethod(d.Name, d);
            if (parent == null) return null;

            // get Child recursively
            if (d.child.Count != 0) {
                foreach (StringData c in d.child) {
                    child.Add(_getXObject(c));
                }
            }

            parent.child = child;

            return parent;
        }

        private static XObj InvokeMethod(string str, StringData d)
        {
            Type thisType = t.GetType();
            MethodInfo theMethod = thisType.GetMethod(
                    "get" + str,
                    BindingFlags.NonPublic | BindingFlags.Static);

            return (theMethod == null)? 
                null: (XObj)theMethod.Invoke(t, new StringData []{d});
        }

        private static XObj getroot(StringData d)
        {
            return new XRoot();
        }

        private static XObj getMesh(StringData d)
        {
            Mesh mesh = new Mesh();
            string str = d.data;
            string[] ver_faces;
            int nver_Index;
            int nfaces_Index;
            string[] vertices;
            string[] faces;
            string[] tmp;

            // split to vertices and faces
            ver_faces = str.Split(new string[] { ";;" }, StringSplitOptions.None);

            // search num index (e.g. num is 92 in "92; 0;1;2;, 3;4;5;, ...")
            nver_Index = ver_faces[0].IndexOf(";");
            nfaces_Index = ver_faces[1].IndexOf(";");

            vertices = ver_faces[0].Substring(nver_Index+1).Split(',');
            faces = ver_faces[1].Substring(nfaces_Index+1).Split(new string[] { ";," }, StringSplitOptions.None);


            // get nVertices and nFaces
            mesh.nVertices = Convert.ToInt32(ver_faces[0].Substring(0, nver_Index)); 
            mesh.nFaces = Convert.ToInt32(ver_faces[1].Substring(0, nfaces_Index));

            // make vertices
            foreach (string vec in vertices) {
                tmp = vec.Split(';');
                mesh.vertices.Add(new Vector3(float.Parse(tmp[0]), float.Parse(tmp[1]), float.Parse(tmp[2]))); 
            }

            // make faces
            foreach (string vec in faces) {
                tmp = vec.Split(';')[1].Split(',');
                if (tmp.Length == 3) {
                    mesh.faces.Add(new Vector3(float.Parse(tmp[0]), float.Parse(tmp[1]), float.Parse(tmp[2]))); 
                } else if (tmp.Length == 4) {
                    mesh.faces.Add(new Vector3(float.Parse(tmp[0]), float.Parse(tmp[1]), float.Parse(tmp[3]))); 
                    mesh.faces.Add(new Vector3(float.Parse(tmp[1]), float.Parse(tmp[2]), float.Parse(tmp[3]))); 
                }
            }

            return mesh;
        }

        private static XObj getMeshNormals(StringData d)
        {
            return getMesh(d);
        }

        private static XObj getMeshTextureCoords(StringData d)
        {
            string str = d.data;
            int n_Index;
            string[] TextureCoords;
            string[] tmp;
            MeshTextureCoords mtc = new MeshTextureCoords();

            // search num index (e.g. num is 92 in "92; 0;2;, 3;4;, ...")
            n_Index = str.IndexOf(";");
            mtc.nTextureCoords = Convert.ToInt32(str.Substring(0, n_Index));

            TextureCoords = str.Substring(n_Index+1).Split(',');
            foreach (string v in TextureCoords) {
                tmp = v.Split(';');
                mtc.textureCoords.Add(new Vector2(float.Parse(tmp[0]), float.Parse(tmp[1])));
            }

            return mtc;
        }

        private static XObj getMeshMaterialList(StringData d)
        {
            string[] str = d.data.Split(';');
            MeshMaterialList mml = new MeshMaterialList();

            mml.nMaterials = Convert.ToInt32(str[0]);
            mml.nFaceIndexes = Convert.ToInt32(str[1]);
            foreach (string s in str[2].Split(',')) {
                mml.faceIndexes.Add(float.Parse(s));
            }

            return mml;
        }

        private static XObj getMaterial(StringData d)
        {
            string[] str = d.data.Split(';');
            Material mt = new Material();

            // Romove empty string
            str = str.Where(x => x.Length!=0).ToArray();

            mt.faceColor = new Vector4(
                    float.Parse(str[0]), 
                    float.Parse(str[1]), 
                    float.Parse(str[2]),
                    float.Parse(str[3]));

            mt.power = float.Parse(str[4]);

            mt.specularColor = new Vector4(
                    float.Parse(str[5]), 
                    float.Parse(str[6]), 
                    float.Parse(str[7]), 
                    0);

            mt.emissiveColor = new Vector4(
                    float.Parse(str[8]), 
                    float.Parse(str[9]), 
                    float.Parse(str[10]),
                    0);

            return mt;
        }

        private static XObj getTextureFilename(StringData d)
        {
            string fname = d.data.Replace("\"", "").Replace(";", "");
            TextureFilename t = new TextureFilename(fname);
            return t;
        }

        private static XObj getMeshVertexColors(StringData d)
        {
            MeshVertexColors mvc = new MeshVertexColors();
            string [] str = d.data.Split(',');
            string [] tmp;

            mvc.nVertexColors = Convert.ToInt32(str[0].Substring(0, str[0].IndexOf(';')));
            str[0] = str[0].Substring(str[0].IndexOf(';')+1);
            foreach (string s in str){
                tmp = s.Split(';');
                mvc.vertexColors.Add(new Vector4(
                            float.Parse(tmp[1]),
                            float.Parse(tmp[2]),
                            float.Parse(tmp[3]),
                            float.Parse(tmp[4])));
            }

            return mvc;
        }
    }
}
