﻿#region

using System.Windows.Forms;
using MMF.CG.MMEEffect;
using SlimDX;
using SlimDX.Direct3D11;
using SlimDX.DXGI;
using Device = SlimDX.Direct3D11.Device;
using Device1 = SlimDX.Direct3D10_1.Device1;
using DriverType = SlimDX.Direct3D10.DriverType;

#endregion

namespace MMF.CG.DeviceManager
{
    /// <summary>
    ///     通常のデバイスの初期化などを行うクラス。
    ///     変更したい場合は継承するといい
    /// </summary>
    public class BasicGraphicDeviceManager : IDeviceManager
    {
        // private RasterizerStateDescription rasterizerStateDesc;

        public FeatureLevel DeviceFeatureLevel { get; private set; }

        /// <summary>
        ///     デバイス
        /// </summary>
        public Device Device { get; private set; }

        /// <summary>
        ///     Device.ImmideateContext
        /// </summary>
        public DeviceContext Context { get; private set; }

        /// <summary>
        ///     Direct2D/DirectWrite用DX10デバイス
        /// </summary>
        public SlimDX.Direct3D10.Device Device10 { get; private set; }

        /// <summary>
        ///     描画に利用しているアダプター
        /// </summary>
        public Adapter CurrentAdapter { get; private set; }

        //public RasterizerStateDescription RasterizerStateDesc
        //{
        //    get { return rasterizerStateDesc; }
        //    set
        //    {
        //        rasterizerStateDesc = value;
        //        if (Context.Rasterizer.State != null && !Context.Rasterizer.State.Disposed) Context.Rasterizer.State.Dispose();
        //        Context.Rasterizer.State = RasterizerState.FromDescription(Device,value);
        //    }
        //}

        ///// <summary>
        /////     リサイズ処理
        ///// </summary>
        ///// <param name="control">適用するコントロールへの参照</param>
        //public void Resize(Control control)
        //{
        //    if (control.Width == 0 || control.Height == 0) return; //フォームがフロート状態になった時一瞬だけ来て、デバイスが作れなくなるのでこの時はなしにする。
        //    CurrentRenderView.Dispose();
        //    CurrentDepthView.Dispose();
        //    SwapChainDescription desc = SwapChain.Description;
        //    SwapChain.ResizeBuffers(desc.BufferCount, control.Width, control.Height, desc.ModeDescription.Format,
        //        desc.Flags);
        //    //深度ステンシルバッファの初期化
        //    using (Texture2D depthBuffer = new Texture2D(Device, getDepthBufferTexture2DDescription(control,desc.SampleDescription)))
        //    {
        //        CurrentDepthView = new DepthStencilView(Device, depthBuffer);
        //    }
        //    //レンダーターゲットの初期化
        //    using (Texture2D renderTexture = Resource.FromSwapChain<Texture2D>(SwapChain, 0))
        //    {
        //        CurrentRenderView = new RenderTargetView(Device, renderTexture);
        //    }
        //    Context = Device.ImmediateContext;
        //    Context.OutputMerger.SetTargets(CurrentDepthView, CurrentRenderView);
        //    Context.Rasterizer.SetViewports(getViewport(control));
        //    //ラスタライザの設定
        //    if (Context.Rasterizer.State != null && !Context.Rasterizer.State.Disposed) Context.Rasterizer.State.Dispose();
        //    Context.Rasterizer.State = RasterizerState.FromDescription(Device,
        //        RasterizerStateDesc);
        //}

        /// <summary>
        ///     デバイスの初期化処理
        /// </summary>
        /// <param name="control">適用するコントロールへの参照</param>
        public void Load()
        {
            //MMGApplication.SetInitializeMessage("DirectX11デバイスの初期化...");
            Factory = new Factory1();
            CurrentAdapter = Factory.GetAdapter1(0);
            //スワップチェーンの初期化
            try
            {
                Device = new Device(CurrentAdapter, DeviceCreationFlags.None,
                    new[] {FeatureLevel.Level_11_0});
            }
            catch (Direct3D11Exception)
            {
                Device = new Device(CurrentAdapter, DeviceCreationFlags.None, new[] {FeatureLevel.Level_10_0});
            }
            DeviceFeatureLevel = Device.FeatureLevel;
            Context = Device.ImmediateContext;
            SampleDescription sampleDesc = new SampleDescription(1, 0);
#if VSG_DEBUG
#else
            Device10 = new Device1(CurrentAdapter, DriverType.Hardware,
                SlimDX.Direct3D10.DeviceCreationFlags.BgraSupport, SlimDX.Direct3D10_1.FeatureLevel.Level_9_3);
#endif
            //SlimDX.Direct2D.Factory factory2d = new SlimDX.Direct2D.Factory(SlimDX.Direct2D.FactoryType.Multithreaded);
            //ラスタライザの設定
            BlendStateDescription blendDesc = new BlendStateDescription();
            blendDesc.AlphaToCoverageEnable = false;
            blendDesc.IndependentBlendEnable = false;
            for (int i = 0; i < blendDesc.RenderTargets.Length; i++)
            {
                blendDesc.RenderTargets[i].BlendEnable = true;
                blendDesc.RenderTargets[i].SourceBlend = BlendOption.SourceAlpha;
                blendDesc.RenderTargets[i].DestinationBlend = BlendOption.InverseSourceAlpha;
                blendDesc.RenderTargets[i].BlendOperation = BlendOperation.Add;
                blendDesc.RenderTargets[i].SourceBlendAlpha = BlendOption.One;
                blendDesc.RenderTargets[i].DestinationBlendAlpha = BlendOption.Zero;
                blendDesc.RenderTargets[i].BlendOperationAlpha = BlendOperation.Add;
                blendDesc.RenderTargets[i].RenderTargetWriteMask = ColorWriteMaskFlags.All;
            }
            Device.ImmediateContext.OutputMerger.BlendState = BlendState.FromDescription(Device, blendDesc);
            MMEEffectManager.IniatializeMMEEffectManager(this);
        }

        public virtual void Dispose()
        {
            if (!Context.Disposed && Context.Rasterizer.State != null && !Context.Rasterizer.State.Disposed)
                Context.Rasterizer.State.Dispose();
            if (Device != null && !Device.Disposed) Device.Dispose();
            if (Device10 != null && !Device10.Disposed) Device10.Dispose();
            if (CurrentAdapter != null && !CurrentAdapter.Disposed) CurrentAdapter.Dispose();
            if (Factory != null && !Factory.Disposed) Factory.Dispose();
        }

        public Factory1 Factory { get; set; }

        protected virtual void PostLoad(Control control)
        {
        }

        /// <summary>
        ///     スワップチェーンの設定を取得します。
        ///     スワップチェーンの設定を変えたい場合は、オーバーライドしてください。
        /// </summary>
        /// <param name="control">適応するコントロールへの参照</param>
        /// <returns>スワップチェーンの設定</returns>
        protected virtual SwapChainDescription getSwapChainDescription(Control control, SampleDescription sampDesc)
        {
            return new SwapChainDescription
            {
                BufferCount = 2,
                Flags = SwapChainFlags.AllowModeSwitch,
                IsWindowed = true,
                ModeDescription = new ModeDescription
                {
                    Format = Format.R8G8B8A8_UNorm,
                    Height = control.Height,
                    Width = control.Width,
                    RefreshRate = new Rational(60, 1)
                },
                OutputHandle = control.Handle,
                SampleDescription = sampDesc,
                SwapEffect = SwapEffect.Discard,
                Usage = Usage.RenderTargetOutput
            };
        }

        /// <summary>
        ///     深度ステンシルバッファの設定を取得します。
        ///     深度ステンシルバッファの設定を変えたい場合はオーバーライドしてください。
        /// </summary>
        /// <param name="control">適用するコントロールへの参照</param>
        /// <returns>深度ステンシルバッファ用のTexture2Dの設定</returns>
        protected virtual Texture2DDescription getDepthBufferTexture2DDescription(Control control,
            SampleDescription desc)
        {
            return new Texture2DDescription
            {
                ArraySize = 1,
                BindFlags = BindFlags.DepthStencil,
                Format = Format.D32_Float,
                Width = control.Width,
                Height = control.Height,
                MipLevels = 1,
                SampleDescription = desc
            };
        }

        /// <summary>
        ///     ビューポートの内容を取得します。
        /// </summary>
        /// <param name="control">適用するコントロールへの参照</param>
        /// <returns>設定するビューポート</returns>
        protected virtual Viewport getViewport(Control control)
        {
            return new Viewport
            {
                Width = control.Width,
                Height = control.Height,
                MaxZ = 1
            };
        }

        /// <summary>
        ///     ラスタライザの設定を取得します。
        /// </summary>
        /// <param name="control">適用するコントロールへの参照</param>
        /// <returns>設定するビューポート</returns>
        protected virtual RasterizerStateDescription getRasterizerStateDescription(Control control)
        {
            return new RasterizerStateDescription
            {
                CullMode = CullMode.Back,
                FillMode = FillMode.Solid,
                DepthBias = 0,
                DepthBiasClamp = 0,
                IsAntialiasedLineEnabled = true,
                IsDepthClipEnabled = true,
                IsFrontCounterclockwise = false,
                IsMultisampleEnabled = true,
                IsScissorEnabled = false,
                SlopeScaledDepthBias = 0
            };
        }
    }
}