﻿using System;
using MMF.CG.MMEEffect.Script;
using MMF.CG.Model.MMD;
using SlimDX.Direct3D11;

namespace MMF.CG.MMEEffect
{
    /// <summary>
    ///     エフェクトのパスを管理するクラス
    /// </summary>
    public class MMEEffectPass
    {
        private RenderContext context;

        public MMEEffectPass(RenderContext context,MMEEffectManager manager, EffectPass pass)
        {
            this.context = context;
            Pass = pass;
            EffectVariable commandAnnotation = EffectParseHelper.getAnnotation(pass, "Script", "string");
            Command = commandAnnotation == null ? "" : commandAnnotation.AsString().GetString();
            if (!pass.VertexShaderDescription.Variable.IsValid)
            {
                //TODO この場合標準シェーダーの頂点シェーダを利用する
            }
            if (!pass.PixelShaderDescription.Variable.IsValid)
            {
                //TODO この場合標準シェーダーのピクセルシェーダを利用する
            }
            ScriptRuntime=new ScriptRuntime(Command,context,manager,this);
        }

        public void Execute(Action<ISubset> drawAction,ISubset subset)
        {
            if (string.IsNullOrWhiteSpace(ScriptRuntime.ScriptCode))
            {
                Pass.Apply(context.DeviceManager.Context);
                drawAction(subset);
            }
            else//スクリプトが存在する場合は処理をスクリプトランタイムに任せる
            {
                ScriptRuntime.Execute(drawAction, subset);
            }
        }

        /// <summary>
        ///     コマンドアノテーション
        /// </summary>
        public string Command { get; private set; }

        public ScriptRuntime ScriptRuntime { get; private set; }

        /// <summary>
        ///     描画に利用されるパス
        /// </summary>
        public EffectPass Pass { get; private set; }
    }
}