﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MMF.CG.MMEEffect.VariableSubscriber.ControlInfoSubscriber;
using MMF.CG.Model;
using MMF.CG.Model.MMD;
using MMF.CG.Skinning;
using SlimDX;
using SlimDX.Direct3D11;

namespace MMF.CG.MMEEffect.VariableSubscriber
{
    internal sealed class ControlObjectSubscriber : SubscriberBase
    {
        internal ControlObjectSubscriber()
        {
        }

        private ControlObjectSubscriber(VariableType type,string itemName,string name,TargetObject target)
        {
            variableType = type;
            this.itemName = itemName;
            this.name = name;
            this.target = target;
        }

        private VariableType variableType;
        private string itemName;
        private string name;
        private TargetObject target;

        private static TargetObject[] NeedFloat = {TargetObject.X,TargetObject.Y,TargetObject.Z,TargetObject.Rx,TargetObject.Ry,TargetObject.Rz,TargetObject.Si,TargetObject.Tr,TargetObject.FaceName };
        private static TargetObject[] NeedFloat3 = { TargetObject.XYZ, TargetObject.Rxyz };


        public override string Semantics
        {
            get { return "CONTROLOBJECT"; }
        }

        public override VariableType[] Types
        {
            get { return new[] { VariableType.Bool, VariableType.Float, VariableType.Float3, VariableType.Float4, VariableType.Float4x4 }; }
        }


        public override SubscriberBase GetSubscriberInstance(EffectVariable variable, RenderContext context, MMEEffectManager effectManager, int semanticIndex)
        {
            VariableType type=0;
            TargetObject target = TargetObject.UnUsed;
            string itemName = null;

            string typeName=variable.GetVariableType().Description.TypeName.ToLower();
            switch (typeName)
            {
                case "bool":
                    type = VariableType.Bool;
                    break;
                case "float":
                    type = VariableType.Float;
                    break;
                case "float3":
                    type = VariableType.Float3;
                    break;
                case "float4":
                    type = VariableType.Float4;
                    break;
                case "float4x4":
                    type = VariableType.Float4x4;
                    break;
                default:
                    break;
            }
            EffectVariable nameVariable = EffectParseHelper.getAnnotation(variable, "name", "string");
            if (nameVariable == null)
            {
                throw new InvalidMMEEffectShaderException(
                    string.Format("定義済みセマンティクス「CONTROLOBJECT」の適用されている変数「{0} {1}:CONTROLOBJECT」に対してはアノテーション「string name」は必須ですが、指定されませんでした。", typeName,variable.Description.Name));
            }
            string name = nameVariable.AsString().GetString();

            EffectVariable itemVariable = EffectParseHelper.getAnnotation(variable, "item", "string");

            if(itemVariable!=null)
            {
                itemName = itemVariable.AsString().GetString();  
                switch(itemName.ToLower())
                {
                    case "x":
                        target = TargetObject.X;
                        break;
                    case "y":
                        target = TargetObject.Y;
                        break;
                    case "z":
                        target = TargetObject.Z;
                        break;
                    case "xyz":
                        target = TargetObject.XYZ;
                        break;
                    case "rx":
                        target = TargetObject.Rx;
                        break;
                    case "ry":
                        target = TargetObject.Ry;
                        break;
                    case "rz":
                        target = TargetObject.Rz;
                        break;
                    case "rxyz":
                        target = TargetObject.Rxyz;
                        break;
                    case "si":
                        target = TargetObject.Si;
                        break;
                    case "tr":
                        target = TargetObject.Tr;
                        break;
                    default:
                        target = type == VariableType.Float ? TargetObject.FaceName : TargetObject.BoneName;
                        break;
                 }
                if (NeedFloat.Contains(target)&&type!=VariableType.Float)
                {
                    throw new InvalidMMEEffectShaderException(
                        string.Format("定義済みセマンティクス「CONTROLOBJECT」の適用されている変数「{0} {1}:CONTROLOBJECT」にはアノテーション「string item=\"{2}\"」が適用されていますが、「{2}」の場合は「float {1}:CONTROLOBJECT」である必要があります。", typeName, variable.Description.Name,itemName));
                }
                if (NeedFloat3.Contains(target) && type != VariableType.Float3)
                {
                    throw new InvalidMMEEffectShaderException(
                        string.Format("定義済みセマンティクス「CONTROLOBJECT」の適用されている変数「{0} {1}:CONTROLOBJECT」にはアノテーション「string item=\"{2}\"」が適用されていますが、「{2}」の場合は「float3 {1}:CONTROLOBJECT」である必要があります。", typeName, variable.Description.Name, itemName));
                
                }
            }
            return new ControlObjectSubscriber(type, itemName, name, target);
        }

        public override void Subscribe(EffectVariable subscribeTo, SubscribeArgument variable)
        {
            
            IEnumerable<IDrawable> targetDrawable = (from drawable in variable.Context.CurrentScreenContext.WorldSpace.DrawableResources where drawable.FileName == name select drawable);
            foreach (var drawable in targetDrawable)
            {
                if (target == TargetObject.UnUsed)
                {
                    switch (variableType)
                    {
                        case VariableType.Float4x4:
                            subscribeTo.AsMatrix().SetMatrix(variable.Context.MatrixManager.WorldMatrixManager.getWorldMatrix(drawable));
                            break;
                        case VariableType.Float3:
                            subscribeTo.AsVector().Set(drawable.Transformer.Position);
                            break;
                        case VariableType.Float4:
                            subscribeTo.AsVector().Set(new Vector4(drawable.Transformer.Position, 1f));
                            break;
                        case VariableType.Float:
                            subscribeTo.AsScalar().Set(drawable.Transformer.Scale.X);
                            break;
                        case VariableType.Bool:
                            subscribeTo.AsScalar().Set(drawable.Visiblity);
                            break;
                        default:
                            break;
                    }
                }
                if (target == TargetObject.BoneName)
                {
                    IEnumerable<Bone> targetBone = (from bone in ((MMDModel)drawable).Skinning.Bone where bone.BoneName == itemName select bone);
                    foreach (var bone in targetBone)
                    {
                        Matrix mat = bone.GlobalPose * variable.Context.MatrixManager.WorldMatrixManager.getWorldMatrix(drawable);
                        switch (variableType)
                        {
                            case VariableType.Float4x4:
                                subscribeTo.AsMatrix().SetMatrix(mat);
                                break;
                            case VariableType.Float3:
                                subscribeTo.AsVector().Set(Vector3.TransformCoordinate(bone.Position,mat));
                                break;
                            case VariableType.Float4:
                                subscribeTo.AsVector().Set(new Vector4(Vector3.TransformCoordinate(bone.Position, mat),1f));
                                break;
                            default:
                                break;
                        }
                        break;
                    }
                }
                break;
            }
        }
    }

        //
}
