﻿using MMF.CG.Camera;
using MMF.CG.Model;
using MMF.CG.Projection;
using MMF.CG.World;
using SlimDX;

namespace MMF.CG
{
    public class MatrixManager
    {
        private readonly IProjectionMatrixProvider projectionMatrixManager;
        private readonly CameraProvider viewMatrixManager;
        private readonly IWorldMatrixProvider worldMatrixManager;

        public MatrixManager(IWorldMatrixProvider world, CameraProvider cam, IProjectionMatrixProvider projection)
        {
            worldMatrixManager = world;
            viewMatrixManager = cam;
            projectionMatrixManager = projection;
        }

        public IWorldMatrixProvider WorldMatrixManager
        {
            get { return worldMatrixManager; }
        }

        public CameraProvider ViewMatrixManager
        {
            get { return viewMatrixManager; }
        }

        public IProjectionMatrixProvider ProjectionMatrixManager
        {
            get { return projectionMatrixManager; }
        }


        public Matrix makeWorldViewProjectionMatrix(Vector3 localScaling, Quaternion localRotation,
            Vector3 localTranslation)
        {
            return worldMatrixManager.getWorldMatrix(localScaling, localRotation, localTranslation)*
                   viewMatrixManager.ViewMatrix*projectionMatrixManager.ProjectionMatrix;
        }

        public Matrix makeWorldViewProjectionMatrix(IDrawable drawable)
        {
            return
                worldMatrixManager.getWorldMatrix(drawable)*viewMatrixManager.ViewMatrix*projectionMatrixManager.ProjectionMatrix;
        }
    }
}