﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Resources;
using MMF.CG.MMEEffect;
using MMF.CG.MMEEffect.VariableSubscriber.ConstantSubscriber.ConstantBufferManager;
using MMDFileParser.PMXModelParser;
using SlimDX;
using SlimDX.Direct3D11;

namespace MMF.CG.Model.MMD
{
    /// <summary>
    /// </summary>
    internal class BasicSubsetManager : ISubsetManager
    {
       // private readonly BasicMaterialConstantBufferManager basicMaterialConstantBuffer = new BasicMaterialConstantBufferManager();
        private Device device;
        //private BasicMaterialConstantBufferInputLayout materialBuffer;
        private ISubresourceLoader subresourceManager;
        private IToonTextureManager toonManager;
        public List<PMXSubset> Subsets { get; private set; }

        public BasicSubsetManager(IDrawable drawable)
        {
            this.Drawable = drawable;
        }

        public void Initialze(object _model, Device device, MMEEffectManager effectManager, ISubresourceLoader subresourceManager,
            IToonTextureManager ToonManager)
        {
            Effect effect = effectManager.EffectFile;
            MMEEffect = effectManager;

            //basicMaterialConstantBuffer.Initialize(device, effect, BasicMaterialConstantBufferInputLayout.SizeInBytes,
            //    materialBuffer);
            toonManager = ToonManager;
            Subsets = new List<PMXSubset>();
            this.device = device;
            this.subresourceManager = subresourceManager;
            ModelData model = (ModelData) _model;
            int vertexSum = 0;
            for (int i = 0; i < model.MaterialList.MaterialCount; i++)
            {
                MaterialData material = model.MaterialList.Materials[i];//サブセットデータを作成する
                PMXSubset dr = new PMXSubset(Drawable,material,i);
                dr.DoCulling = !material.bitFlag.HasFlag(RenderFlag.CullNone);
                dr.VertexCount = material.VertexNumber/3;
                dr.StartIndex = vertexSum;
                if (material.textureIndex >= toonManager.ResourceViews.Length)
                {
                    if (toonManager.ResourceViews.Length == 0)
                    {
                        int index = ToonManager.LoadToon(model.TextureList.TexturePathes[material.textureIndex]);
                        dr.MaterialInfo.ToonTexture = ToonManager.ResourceViews[index];
                        dr.MaterialInfo.IsToonUsed = false;
                    }
                    else
                    {
                        dr.MaterialInfo.ToonTexture = ToonManager.ResourceViews[0];
                        dr.MaterialInfo.IsToonUsed = false;
                    }
                }
                else
                {
                    if (material.ShareToonFlag == 1)
                    {
                        dr.MaterialInfo.ToonTexture = ToonManager.ResourceViews[material.textureIndex + 1];
                        dr.MaterialInfo.IsToonUsed = true;
                    }
                    else if (material.textureIndex != model.GetTextureNoReferenceCode())
                    {
                        if (model.TextureList.TexturePathes.Count < material.textureIndex + 1)
                        {
                            dr.MaterialInfo.ToonTexture = ToonManager.ResourceViews[0];
                            dr.MaterialInfo.IsToonUsed = false;
                        }
                        else
                        {
                            int index = ToonManager.LoadToon(model.TextureList.TexturePathes[material.textureIndex]);
                            dr.MaterialInfo.ToonTexture = ToonManager.ResourceViews[index];
                            dr.MaterialInfo.IsToonUsed = true;
                        }
                    }
                    else
                    {
                        dr.MaterialInfo.ToonTexture = ToonManager.ResourceViews[0];
                        dr.MaterialInfo.IsToonUsed = false;
                    }
                }
                vertexSum += material.VertexNumber;

                //テクスチャの読み込み
                if (material.TextureTableReferenceIndex != model.GetTextureNoReferenceCode())
                {
                    dr.MaterialInfo.MaterialTexture = getSubresourceById(model.TextureList.TexturePathes[material.TextureTableReferenceIndex]);
                }
                //スフィアマップの読み込み
                if (material.SphereTextureTableReferenceIndex != model.GetTextureNoReferenceCode())
                {
                    dr.MaterialInfo.SphereMode = material.SphereMode;
                    dr.MaterialInfo.MaterialSphereMap =
                        getSubresourceById(model.TextureList.TexturePathes[material.SphereTextureTableReferenceIndex]);
                }
                Subsets.Add(dr);
            }
        }

        public void ResetEffect(MMEEffectManager effect)
        {
            this.MMEEffect=effect;
        }


        public void DrawAll()
        {
            for (int i = 0; i < Subsets.Count; i++)
            {
                UpdateConstantByMaterial(Subsets[i]);
                MMEEffect.ApplyEffectPass(Subsets[i],MMEEffectPassType.Object, (subset) => device.ImmediateContext.DrawIndexed(3 * subset.VertexCount, subset.StartIndex, 0));
            }
        }

        public void DrawEdges()
        {
            foreach (PMXSubset variable in from subset in Subsets where  subset.MaterialInfo.isEdgeEnable select subset)
            {
                UpdateConstantByMaterial(variable);
                MMEEffect.ApplyEffectPass(variable, MMEEffectPassType.Edge, (subset) => device.ImmediateContext.DrawIndexed(3 * subset.VertexCount, subset.StartIndex, 0));
            }
        }

        public void DrawGroundShadow()
        {
            foreach (PMXSubset variable in from subset in Subsets where subset.MaterialInfo.isGroundShadowEnable select subset)
            {
                UpdateConstantByMaterial(variable);
                MMEEffect.ApplyEffectPass(variable, MMEEffectPassType.Shadow, (subset) => device.ImmediateContext.DrawIndexed(3 * subset.VertexCount, subset.StartIndex, 0));
            }
        }

        public int SubsetCount
        {
            get
            {
                return Subsets.Count;
            }
        }

        public void Dispose()
        {
            foreach (PMXSubset drawableResource in Subsets)
            {
                drawableResource.Dispose();
            }
        }


        private void UpdateConstantByMaterial(PMXSubset subset)
        {
            MMEEffect.ApplyAllMaterialVariables(subset.MaterialInfo);
        }

        private ShaderResourceView getSubresourceById(string p)
        {
            using (Stream stream = subresourceManager.getSubresourceByName(p))
            {
                return ShaderResourceView.FromStream(device, stream, (int) stream.Length);
            }
        }

        public IDrawable Drawable { get; set; }

        public MMEEffectManager MMEEffect { get; set; }
    }
}