﻿using System.Collections.Generic;
using System.IO;
using MMF.CG.Utility;
using SlimDX.Direct3D11;

namespace MMF.CG.Model.MMD
{
    internal class BasicToonTextureManager : IToonTextureManager
    {
        private readonly List<ShaderResourceView> resourceViewsList = new List<ShaderResourceView>();
        private Device _device;

        private ISubresourceLoader _subresourceManager;

        public ShaderResourceView[] ResourceViews
        {
            get { return resourceViewsList.ToArray(); }
        }

        public void Initialize(Device device, ISubresourceLoader subresourceManager)
        {
            _device = device;
            _subresourceManager = subresourceManager;
            for (int i = 0; i < 11; i++)
            {
                string path = CGHelper.ToonDir + string.Format("toon{0}.bmp", i);
                if (File.Exists(path))
                    resourceViewsList.Add(ShaderResourceView.FromFile(device, path));
                else
                {
                    
                }
            }
        }

        public int LoadToon(string path)
        {
            using (Stream stream = _subresourceManager.getSubresourceByName(path))
            {
                resourceViewsList.Add(ShaderResourceView.FromStream(_device, stream, (int) stream.Length));
                return resourceViewsList.Count - 1;
            }
        }

        public void Dispose()
        {
            foreach (ShaderResourceView shaderResourceView in resourceViewsList)
            {
                shaderResourceView.Dispose();
            }
        }
    }
}