﻿using System.IO;
using MMF.CG.Skinning;
using MMDFileParser.PMXModelParser;

namespace MMF.CG.Model.MMD
{
    public class MMDModelWithPhysics : MMDModel
    {
        public MMDModelWithPhysics(ModelData modeldata, ISubresourceLoader subResourceLoader, string filename)
            : base(modeldata, subResourceLoader, filename)
        {
        }

        public new static MMDModel OpenLoad(string filePath, RenderContext context)
        {
            MMDModelWithPhysics model = (MMDModelWithPhysics) FromFile(filePath);
            model.Load(context);
            return model;
        }

        public new static MMDModel FromFile(string filePath)
        {
            string folder = Path.GetDirectoryName(filePath);
            return FromFile(filePath, folder);
        }

        public new static MMDModel FromFile(string filePath, string textureFolder)
        {
            return FromFile(filePath, new BasicSubresourceLoader(textureFolder));
        }

        public new static MMDModel FromFile(string filePath, ISubresourceLoader loader)
        {
            using (FileStream fs = File.OpenRead(filePath))
            {
                return new MMDModelWithPhysics(ModelData.GetModel(fs), loader, Path.GetFileName(filePath));
            }
        }

        public override void Dispose()
        {
            Skinning.Dispose();
            base.Dispose();
        }

        protected override ISkinningProvider InitializeSkinning()
        {
            SkeletonWithPhysics skel = new SkeletonWithPhysics(Model);
            return skel;
        }
    }
}