﻿using System.Collections.Generic;
using System.Linq;
using MMF.CG.Motion;
using MMF.CG.Skinning;
using MMDFileParser.PMXModelParser;
using MMDFileParser.PMXModelParser.MorphOffset;
using SlimDX;

namespace MMF.CG.Model.MMD.Morph
{
    internal class BoneMorphProvider : IMMDMorphProvider
    {
        public Dictionary<string, BoneMorphData> MorphList = new Dictionary<string, BoneMorphData>();
        private ISkinningProvider skinningProvider;

        public BoneMorphProvider(MMDModel model)
        {
            skinningProvider = model.Skinning;
            foreach (MorphData morphData in model.Model.MorphList.Morphes)
            {
                if (morphData.type == MorphType.Bone)
                {
                    MorphList.Add(morphData.MorphName, new BoneMorphData(morphData));
                }
            }
        }

        public void ApplyMorphProgress(float frame, IEnumerable<FaceMotion> faces)
        {
            foreach (FaceMotion faceMotion in faces)
            {
                SetMorphProgress(faceMotion.Lerp(frame), faceMotion.Morphname);
            }
        }

        public bool ApplyMorphProgress(float progress, string morphName)
        {
            return SetMorphProgress(progress, morphName);
        }

        public void UpdateFrame()
        {
            
        }

        private bool SetMorphProgress(float progress, string morphName)
        {
            if (!MorphList.ContainsKey(morphName)) return false;
            BoneMorphData data = MorphList[morphName];
            foreach (BoneMorphOffset boneMorphOffset in data.BoneMorphs)
            {
                Quaternion rot=new Quaternion(boneMorphOffset.QuantityOfRotating.X,boneMorphOffset.QuantityOfRotating.Y,boneMorphOffset.QuantityOfRotating.Z,boneMorphOffset.QuantityOfRotating.W);
                skinningProvider.Bone[boneMorphOffset.BoneIndex].Rotation *= rot;
                skinningProvider.Bone[boneMorphOffset.BoneIndex].Translation += boneMorphOffset.QuantityOfMoving;
            }
            return true;
        }
    }
}