﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using BulletSharp;
using MMF.CG.MMEEffect;
using MMF.CG.Motion;
using MMDFileParser.PMXModelParser;
using MMDFileParser.PMXModelParser.MorphOffset;

namespace MMF.CG.Model.MMD.Morph
{
    public class GroupMorphProvider:IMMDMorphProvider
    {
        public GroupMorphProvider(MMDModel model,IMMDMorphManager morph)
        {
            this.morphManager = morph;
            int i = 0;
            foreach (MorphData morphData in model.Model.MorphList.Morphes)
            {
                if (morphData.type == MorphType.Group)
                {
                    Morphs.Add(morphData.MorphName,new GroupMorphData(morphData));
                }
                morphNameList.Add(i,morphData.MorphName);
                i++;
            }
        }

        private IMMDMorphManager morphManager;

        private Dictionary<int,string> morphNameList=new Dictionary<int, string>(); 

        public Dictionary<string,GroupMorphData> Morphs=new Dictionary<string, GroupMorphData>(); 

        public void ApplyMorphProgress(float frame, IEnumerable<FaceMotion> faces)
        {
            foreach (FaceMotion faceMotion in faces)
            {
                SetMorphProgress(faceMotion.Lerp(frame), faceMotion.Morphname);
            }
        }

        public bool ApplyMorphProgress(float progress, string morphName)
        {
            return SetMorphProgress(progress, morphName);
        }

        public void UpdateFrame()
        {
            
        }

        private bool SetMorphProgress(float progress, string morphName)
        {
            if (!Morphs.ContainsKey(morphName)) return false;
            GroupMorphData data = Morphs[morphName];
            foreach (GroupMorphOffset groupMorphOffset in data.MorphOffsets)
            {
                string targetMorph = morphNameList[groupMorphOffset.MorphIndex];
                if(morphName.Equals(targetMorph))throw new InvalidOperationException("グループモーフに自身のモーフが指定されています。");
                morphManager.ApplyMorphProgress(progress*groupMorphOffset.MorphRatio,targetMorph);
            }
            return true;
        }
    }

    public class GroupMorphData
    {
        public List<GroupMorphOffset> MorphOffsets=new List<GroupMorphOffset>();

        public GroupMorphData(MorphData data)
        {
            foreach (MorphOffsetBase morphOffsetBase in data.MorphOffsetes)
            {
                MorphOffsets.Add((GroupMorphOffset)morphOffsetBase);
            }
        }
    }
}
