﻿using System.Collections.Generic;
using MMF.CG.Motion;
using MMDFileParser.PMXModelParser;
using SlimDX.Direct3D9;

namespace MMF.CG.Model.MMD.Morph
{
    /// <summary>
    /// PMX標準のモーフを管理するクラス
    /// </summary>
    public class MMDMorphManager : IMMDMorphManager
    {
        public MMDMorphManager(MMDModel model)
        {
            MMDMorphs.Add(new VertexMorphProvider(model.Model,model.BufferManager));
            MMDMorphs.Add(new BoneMorphProvider(model));
            MMDMorphs.Add(new MaterialMorphProvider(model));
            MMDMorphs.Add(new GroupMorphProvider(model,this));
            MMDMorphs.Add(new UVMorphProvider(model,MorphType.UV));
            MMDMorphs.Add(new UVMorphProvider(model,MorphType.UV_Additional1));
            MMDMorphs.Add(new UVMorphProvider(model,MorphType.UV_Additional2));
            MMDMorphs.Add(new UVMorphProvider(model, MorphType.UV_Additional3));
            MMDMorphs.Add(new UVMorphProvider(model, MorphType.UV_Additional4));
        }

        public List<IMMDMorphProvider> MMDMorphs=new List<IMMDMorphProvider>();

        /// <summary>
        /// モーフのモーションから再生
        /// </summary>
        /// <param name="frame"></param>
        /// <param name="morphs"></param>
        public void ApplyMorphProgress(float frame, IEnumerable<FaceMotion> morphs)
        {
            foreach (IMMDMorphProvider mmdMorphManager in MMDMorphs)
            {
                mmdMorphManager.ApplyMorphProgress(frame, morphs);
            }
        }

        /// <summary>
        /// 指定したモーフのフレームをセットする
        /// </summary>
        /// <param name="frame"></param>
        /// <param name="morphName"></param>
        public void ApplyMorphProgress(float frame, string morphName)
        {
            foreach (IMMDMorphProvider mmdMorphManager in MMDMorphs)
            {
                if(mmdMorphManager.ApplyMorphProgress(frame,morphName))return;
            }
        }

        public void UpdateFrame()
        {
            foreach (var mmdMorphProvider in MMDMorphs)
            {
                mmdMorphProvider.UpdateFrame();
            }
        }
    }
}
