﻿using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MMF.CG.Model.MMD.Morph;
using MMF.CG.Motion;
using MMDFileParser.PMXModelParser;
using MMDFileParser.PMXModelParser.MorphOffset;
using SlimDX;
using SlimDX.Direct3D11;

namespace MMF.CG.Model.MMD
{
    /// <summary>
    /// 頂点モーフを管理するクラス
    /// </summary>
    public class VertexMorphProvider : IMMDMorphProvider
    {
        public Dictionary<string,VertexMorphData> MorphList=new Dictionary<string,VertexMorphData>();

        private HashSet<int> movedVertex=new HashSet<int>();

        private ModelData model;

        public VertexMorphProvider(ModelData model,IBufferManager bufManager)
        {
            this.model = model;
            Buffermanager = bufManager;
            foreach (MorphData morphData in model.MorphList.Morphes)
            {
                if (morphData.type == MorphType.Vertex)
                {
                    MorphList.Add(morphData.MorphName,new VertexMorphData(morphData));
                }
            }
        }

        public void ApplyMorphProgress(float frame, IEnumerable<FaceMotion> faces)
        {
            
            foreach (FaceMotion motion in faces)
            {
                SetMorphProgress(motion.Lerp(frame), motion.Morphname);
            }
        }

        public bool ApplyMorphProgress(float progress, string morphName)
        {
            return SetMorphProgress(progress, morphName);
        }

        public void UpdateFrame()
        {
            ResetVertexPosition();
        }


        private void ResetVertexPosition()
        {
            foreach (int i in movedVertex)
            {
                VertexData vertexData = model.VertexList.Vertexes[i];
                Buffermanager.InputVerticies[i].Position = new Vector4(vertexData.Position, 1f);
            }
            movedVertex=new HashSet<int>();
        }


        private bool SetMorphProgress(float progress, string morphName)
        {
            if(!MorphList.ContainsKey(morphName))return false;
            VertexMorphData data = MorphList[morphName];
            foreach (VertexMorphOffset vertexMorph in data.MorphOffsets)
            {
                movedVertex.Add(vertexMorph.VertexIndex);
                Buffermanager.InputVerticies[vertexMorph.VertexIndex].Position += new Vector4(vertexMorph.PositionOffset*progress,0);
            }
            Buffermanager.NeedReset = true;
            return true;
        }

        private IBufferManager Buffermanager { get; set; }
    }
}
