﻿using System;
using MMF.CG.Model.MMD;
using MMF.CG.Skinning;
using MMDFileParser.MotionParser;

namespace MMF.CG.Motion
{
    public interface IMotionProvider
    {
        /// <summary>
        ///     モーションデータ
        /// </summary>
        MotionData Motion { get; }

        bool isLoaded { get; }

        /// <summary>
        ///     現在のフレーム
        /// </summary>
        float CurrentFrame { get; set; }

        /// <summary>
        ///     このモーションにおける最終フレーム
        /// </summary>
        int FinalFrame { get;}

        /// <summary>
        ///     モーションを読み込みます。
        /// </summary>
        /// <param name="Skinning">割り当てるスキニング</param>
        void LoadMotion(ISkinningProvider skinning);

        /// <summary>
        ///     モーションを再生します。
        /// </summary>
        /// <param name="frame">再生フレームの最初</param>
        /// <param name="action">モーション終了後の挙動</param>
        void Start(float frame, ActionAfterMotion action);

        /// <summary>
        ///     モーションを停止します。
        /// </summary>
        void Stop();

        /// <summary>
        ///     モーションを１フレーム進めます。
        /// </summary>
        void Tick();

        /// <summary>
        ///     モーションが終了したことを表します。
        /// </summary>
        event EventHandler<ActionAfterMotion> MotionFinished;

        event EventHandler<EventArgs> FrameTicked;
    }

    /// <summary>
    ///     モーション再生終了後の挙動を指定する列挙体
    /// </summary>
    public enum ActionAfterMotion
    {
        Nothing,
        Replay
    }
}