﻿using System.Collections.Generic;
using MMF.CG.Utility;
using MMDFileParser.MotionParser;
using SlimDX;

namespace MMF.CG.Skinning
{
    /// <summary>
    ///     ボーンのモーション管理
    /// </summary>
    internal class BoneMotion
    {
        /// <summary>
        /// モーションのフレームごとのデータ
        /// </summary>
        public List<BoneFrameData> Frames { get; private set; }

        public BoneMotion(int index)
        {
            BoneIndex = index;
            Frames = new List<BoneFrameData>();
        }

        /// <summary>
        ///     割り当てられているボーンのインデックス
        /// </summary>
        public int BoneIndex { get; private set; }

        private int lastCount=0 ;

        /// <summary>
        /// 指定したボーンを補完する
        /// </summary>
        /// <param name="target">補完するボーン</param>
        /// <param name="frame">フレーム位置</param>
        public void Lerp(Bone target, float frame)
        {
            for (int j = lastCount; j < Frames.Count - 1; j++)
            {
                if (Frames[j].FrameNumber < frame && Frames[j + 1].FrameNumber >= frame)
                {
                    //フレームが挟まれている時
                    uint frameMargin = Frames[j + 1].FrameNumber - Frames[j].FrameNumber;
                    float progress = (frame - Frames[j].FrameNumber) / (float)frameMargin;
                    float ProgX, ProgY, ProgZ, ProgR;
                    ProgX = Frames[j].Curves[0].Evaluate(progress);
                    ProgY = Frames[j].Curves[1].Evaluate(progress);
                    ProgZ = Frames[j].Curves[2].Evaluate(progress);
                    ProgR = Frames[j].Curves[3].Evaluate(progress);

                    target.Translation = CGHelper.ComplementTranslate(Frames[j], Frames[j + 1],
                        new Vector3(ProgX, ProgY, ProgZ));
                    target.Rotation = CGHelper.ComplementRotateQuaternion(Frames[j], Frames[j + 1], ProgR);
                    lastCount = j;
                    return;
                }
            }
            for (int j = 0; j < Frames.Count - 1; j++)//最初から探索
            {
                if (Frames[j].FrameNumber < frame && Frames[j + 1].FrameNumber >= frame)
                {
                    //フレームが挟まれている時
                    uint frameMargin = Frames[j + 1].FrameNumber - Frames[j].FrameNumber;
                    float progress = (frame - Frames[j].FrameNumber) / (float)frameMargin;
                    float ProgX, ProgY, ProgZ, ProgR;
                    ProgX = Frames[j].Curves[0].Evaluate(progress);
                    ProgY = Frames[j].Curves[1].Evaluate(progress);
                    ProgZ = Frames[j].Curves[2].Evaluate(progress);
                    ProgR = Frames[j].Curves[3].Evaluate(progress);

                    target.Translation = CGHelper.ComplementTranslate(Frames[j], Frames[j + 1],
                        new Vector3(ProgX, ProgY, ProgZ));
                    target.Rotation = CGHelper.ComplementRotateQuaternion(Frames[j], Frames[j + 1], ProgR);
                    lastCount = j;
                    return;
                }
            }
        }
    }
}