﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using MMF.CG.Model.MMD;
using MMF.CG.Model.MMD.Morph;
using SlimDX;

namespace MMF.CG.Skinning
{
    public class ManualTransformUpdater:ITransformUpdater
    {
        private  Dictionary<string,BoneTransformer> updaters=new Dictionary<string, BoneTransformer>();

        private Dictionary<string,MorphTransformer> morphUpdaters=new Dictionary<string, MorphTransformer>();

        private MMDModel model;

        public ManualTransformUpdater(MMDModel model)
        {
            this.model = model;
        }


        bool ITransformUpdater.UpdateTransform(ISkinningProvider prov, IMMDMorphManager morph)
        {
            foreach (var boneTransformer in updaters)
            {
                Bone bone = prov.BoneDictionary[boneTransformer.Key];
                bone.Rotation *= boneTransformer.Value.Rotation;
                bone.Translation += boneTransformer.Value.Translation;
            }
            foreach (var morphTransformer in morphUpdaters)
            {
                morph.ApplyMorphProgress(morphTransformer.Value.MorphValue,morphTransformer.Key);
            }
            return true;
        }

        public BoneTransformer getBoneTransformer(string boneName)
        {
            BoneTransformer transformer;
            if(!model.Skinning.BoneDictionary.ContainsKey(boneName))throw new InvalidOperationException("そのような名前のボーンは存在しません。");
            if (updaters.ContainsKey(boneName)) return updaters[boneName];
            else transformer=new BoneTransformer(boneName,Quaternion.Identity,Vector3.Zero);
            updaters.Add(transformer.BoneName,transformer);
            return transformer;
        }

        public MorphTransformer getMorphTransformer(string morphName)
        {
            MorphTransformer transformer;
            if (morphUpdaters.ContainsKey(morphName)) return morphUpdaters[morphName];
            else transformer=new MorphTransformer(morphName);
            morphUpdaters.Add(transformer.MorphName,transformer);
            return transformer;
        }
    }

    public class MorphTransformer
    {
        public string MorphName { get; private set; }

        public float MorphValue { get; set; }

        public MorphTransformer(string morphName)
        {
            MorphName = morphName;
        }
    }
}
